@@ -42,6 +42,7 @@ OVN_QOS_POLICY_EXT_ID_KEY = 'neutron:qos_policy_id'
 OVN_SG_IDS_EXT_ID_KEY = 'neutron:security_group_ids'
 OVN_DEVICE_OWNER_EXT_ID_KEY = 'neutron:device_owner'
 OVN_LIVENESS_CHECK_EXT_ID_KEY = 'neutron:liveness_check_at'
+METADATA_LIVENESS_CHECK_EXT_ID_KEY = 'neutron:metadata_liveness_check_at'
 OVN_PORT_BINDING_PROFILE = portbindings.PROFILE
 OVN_PORT_BINDING_PROFILE_PARAMS = [{'parent_name': six.string_types,
                                     'tag': six.integer_types},
@@ -53,6 +54,7 @@ OVN_GATEWAY_CHASSIS_KEY = 'redirect-chassis'
 OVN_CHASSIS_REDIRECT = 'chassisredirect'
 OVN_GATEWAY_NAT_ADDRESSES_KEY = 'nat-addresses'
 OVN_DROP_PORT_GROUP_NAME = 'neutron_pg_drop'
+OVN_ROUTER_PORT_GW_MTU_OPTION = 'gateway_mtu'
 
 OVN_PROVNET_PORT_NAME_PREFIX = 'provnet-'
 
@@ -155,9 +157,27 @@ MAINTENANCE_DELETE_TYPE_ORDER = {
 # peer router port (connecting to the logical router).
 DEFAULT_ADDR_FOR_LSP_WITH_PEER = 'router'
 
+# FIP ACTIONS
+FIP_ACTION_ASSOCIATE = 'fip_associate'
+FIP_ACTION_DISASSOCIATE = 'fip_disassociate'
+
 # Loadbalancer constants
 LRP_PREFIX = "lrp-"
 LB_VIP_PORT_PREFIX = "ovn-lb-vip-"
+LB_EXT_IDS_LS_REFS_KEY = 'ls_refs'
+LB_EXT_IDS_LR_REF_KEY = 'lr_ref'
+LB_EXT_IDS_POOL_PREFIX = 'pool_'
+LB_EXT_IDS_LISTENER_PREFIX = 'listener_'
+LB_EXT_IDS_MEMBER_PREFIX = 'member_'
+LB_EXT_IDS_VIP_KEY = 'neutron:vip'
+LB_EXT_IDS_VIP_FIP_KEY = 'neutron:vip_fip'
+LB_EXT_IDS_VIP_PORT_ID_KEY = 'neutron:vip_port_id'
+
+# Hash Ring constants
+HASH_RING_NODES_TIMEOUT = 60
+HASH_RING_TOUCH_INTERVAL = 30
+HASH_RING_CACHE_TIMEOUT = 30
+HASH_RING_ML2_GROUP = 'mechanism_driver'
 
 # Maximum chassis count where a gateway port can be hosted
 MAX_GW_CHASSIS = 5
@@ -166,3 +186,6 @@ UNKNOWN_ADDR = 'unknown'
 
 # TODO(lucasagomes): Create constants for other LSP types
 LSP_TYPE_LOCALNET = 'localnet'
+LSP_TYPE_VIRTUAL = 'virtual'
+LSP_OPTIONS_VIRTUAL_PARENTS_KEY = 'virtual-parents'
+LSP_OPTIONS_VIRTUAL_IP_KEY = 'virtual-ip'
