@@ -17,17 +17,17 @@ import inspect
 import threading
 
 from futurist import periodics
-from neutron.common import config as n_conf
+from neutron_lib.api.definitions import external_net
 from neutron_lib import constants as n_const
 from neutron_lib import context as n_context
 from neutron_lib import exceptions as n_exc
-from neutron_lib import worker
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
 
 from networking_ovn.common import config as ovn_conf
 from networking_ovn.common import constants as ovn_const
+from networking_ovn.db import hash_ring as db_hash_ring
 from networking_ovn.db import maintenance as db_maint
 from networking_ovn.db import revision as db_rev
 from networking_ovn import ovn_db_sync
@@ -40,27 +40,6 @@ INCONSISTENCY_TYPE_CREATE_UPDATE = 'create/update'
 INCONSISTENCY_TYPE_DELETE = 'delete'
 
 
-class MaintenanceWorker(worker.BaseWorker):
-
-    def start(self):
-        super(MaintenanceWorker, self).start()
-        # NOTE(twilson) The super class will trigger the post_fork_initialize
-        # in the driver, which starts the connection/IDL notify loop which
-        # keeps the process from exiting
-
-    def stop(self):
-        """Stop service."""
-        super(MaintenanceWorker, self).stop()
-
-    def wait(self):
-        """Wait for service to complete."""
-        super(MaintenanceWorker, self).wait()
-
-    @staticmethod
-    def reset():
-        n_conf.reset_service()
-
-
 class MaintenanceThread(object):
 
     def __init__(self):
@@ -452,7 +431,47 @@ class DBInconsistenciesPeriodics(object):
                 elif not type_ and ovn_const.UNKNOWN_ADDR in addresses:
                     addresses.remove(ovn_const.UNKNOWN_ADDR)
 
+            addresses = port.addresses
+            type_ = port.type.strip()
+            if not port.port_security:
+                if not type_ and ovn_const.UNKNOWN_ADDR not in addresses:
+                    addresses.append(ovn_const.UNKNOWN_ADDR)
+                elif type_ and ovn_const.UNKNOWN_ADDR in addresses:
+                    addresses.remove(ovn_const.UNKNOWN_ADDR)
+            else:
+                if type_ and ovn_const.UNKNOWN_ADDR in addresses:
+                    addresses.remove(ovn_const.UNKNOWN_ADDR)
+                elif not type_ and ovn_const.UNKNOWN_ADDR in addresses:
+                    addresses.remove(ovn_const.UNKNOWN_ADDR)
+
             self._nb_idl.lsp_set_addresses(
                 port.name, addresses=addresses).execute(check_error=True)
 
         raise periodics.NeverAgain()
+
+    # A static spacing value is used here, but this method will only run
+    # once per lock due to the use of periodics.NeverAgain().
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def check_for_fragmentation_support(self):
+        if not self.has_lock:
+            return
+
+        context = n_context.get_admin_context()
+        for net in self._ovn_client._plugin.get_networks(
+                context, {external_net.EXTERNAL: [True]}):
+            self._ovn_client.set_gateway_mtu(context, net)
+
+        raise periodics.NeverAgain()
+
+
+class HashRingHealthCheckPeriodics(object):
+
+    def __init__(self, group):
+        self._group = group
+
+    @periodics.periodic(spacing=ovn_const.HASH_RING_TOUCH_INTERVAL)
+    def touch_hash_ring_nodes(self):
+        # NOTE(lucasagomes): Note that we do not rely on the OVSDB lock
+        # here because we want the maintenance tasks from each instance to
+        # execute this task.
+        db_hash_ring.touch_nodes_from_host(self._group)
