@@ -16,7 +16,7 @@ import os
 import re
 
 import netaddr
-
+from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import extra_dhcp_opt as edo_ext
 from neutron_lib.api.definitions import l3
 from neutron_lib.api.definitions import port_security as psec
@@ -338,7 +338,7 @@ def get_lrouter_non_gw_routes(ovn_router):
         for route in getattr(ovn_router, 'static_routes', []):
             external_ids = getattr(route, 'external_ids', {})
             if strutils.bool_from_string(
-                external_ids.get(constants.OVN_ROUTER_IS_EXT_GW, 'false')):
+                    external_ids.get(constants.OVN_ROUTER_IS_EXT_GW, 'false')):
                 continue
 
             routes.append({'destination': route.ip_prefix,
@@ -421,36 +421,8 @@ def is_gateway_chassis_invalid(chassis_name, gw_chassis,
     return False
 
 
-def is_neutron_dhcp_agent_port(port):
-    """Check if the given DHCP port belongs to Neutron DHCP agents
-
-    The DHCP ports with the device_id equals to 'reserved_dhcp_port'
-    or starting with the word 'dhcp' belongs to the Neutron DHCP agents.
-    """
-    return (port['device_owner'] == const.DEVICE_OWNER_DHCP and
-            (port['device_id'] == const.DEVICE_ID_RESERVED_DHCP_PORT or
-             port['device_id'].startswith('dhcp')))
-
-
-def is_gateway_chassis(chassis):
-    """Check if the given chassis is a gateway chassis"""
-    external_ids = getattr(chassis, 'external_ids', {})
-    return ('enable-chassis-as-gw' in external_ids.get(
-        'ovn-cms-options', '').split(','))
-
-
-def get_port_id_from_gwc_row(row):
-    """Return a port_id from gwc row
-
-    The Gateway_Chassis row stores router port_id in
-    the row name attribute:
-
-    <prefix>-<port_id>_<chassis_id>
-
-    :param row: A Gateway_Chassis table row.
-    :returns: String containing router port_id.
-    """
-    return constants.RE_PORT_FROM_GWC.search(row.name).group(2)
+def is_provider_network(network):
+    return external_net.EXTERNAL in network
 
 
 def compute_address_pairs_diff(ovn_port, neutron_port):
@@ -461,3 +433,14 @@ def compute_address_pairs_diff(ovn_port, neutron_port):
     added = set(neutron_ap) - set(ovn_ap)
     removed = set(ovn_ap) - set(neutron_ap)
     return AddrPairsDiff(added, removed, changed=any(added or removed))
+
+
+def is_neutron_dhcp_agent_port(port):
+    """Check if the given DHCP port belongs to Neutron DHCP agents
+
+    The DHCP ports with the device_id equals to 'reserved_dhcp_port'
+    or starting with the word 'dhcp' belongs to the Neutron DHCP agents.
+    """
+    return (port['device_owner'] == const.DEVICE_OWNER_DHCP and
+            (port['device_id'] == const.DEVICE_ID_RESERVED_DHCP_PORT or
+             port['device_id'].startswith('dhcp')))
