@@ -674,27 +674,50 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def get_port_group(self, pg_name):
         if uuidutils.is_uuid_like(pg_name):
             pg_name = utils.ovn_port_group_name(pg_name)
-
-        for pg in self._tables['Port_Group'].rows.values():
-            if pg.name == pg_name:
-                return pg
+        try:
+            for pg in self._tables['Port_Group'].rows.values():
+                if pg.name == pg_name:
+                    return pg
+        except KeyError:
+            # TODO(dalvarez): This except block is added for backwards compat
+            # with old OVN schemas (<=2.9) where Port Groups are not present.
+            # This (and other conditional code around this feature) shall be
+            # removed at some point.
+            return
 
     def get_port_groups(self):
         port_groups = {}
-        for row in self._tables['Port_Group'].rows.values():
-            name = getattr(row, 'name')
-            if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
-               name == ovn_const.OVN_DROP_PORT_GROUP_NAME):
-                continue
-            data = {}
-            for row_key in getattr(row, "_data", {}):
-                data[row_key] = getattr(row, row_key)
-            port_groups[name] = data
+        try:
+            for row in self._tables['Port_Group'].rows.values():
+                name = getattr(row, 'name')
+                if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
+                   name == ovn_const.OVN_DROP_PORT_GROUP_NAME):
+                    continue
+                data = {}
+                for row_key in getattr(row, "_data", {}):
+                    data[row_key] = getattr(row, row_key)
+                port_groups[name] = data
+        except KeyError:
+            # TODO(dalvarez): This except block is added for backwards compat
+            # with old OVN schemas (<=2.9) where Port Groups are not present.
+            # This (and other conditional code around this feature) shall be
+            # removed at some point.
+            pass
         return port_groups
 
     def check_liveness(self):
         return cmd.CheckLivenessCommand(self)
 
+    def set_lswitch_port_to_virtual_type(self, lport_name, vip,
+                                         virtual_parent, if_exists=True):
+        return cmd.SetLSwitchPortToVirtualTypeCommand(
+            self, lport_name, vip, virtual_parent, if_exists)
+
+    def unset_lswitch_port_to_virtual_type(self, lport_name,
+                                           virtual_parent, if_exists=True):
+        return cmd.UnsetLSwitchPortToVirtualTypeCommand(
+            self, lport_name, virtual_parent, if_exists)
+
 
 class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
     def __init__(self, connection):
