@@ -14,11 +14,6 @@
 
 import mock
 
-from networking_ovn.common import constants as ovn_const
-from networking_ovn.common import utils as ovn_utils
-from networking_ovn.l3 import l3_ovn_scheduler as l3_sched
-from networking_ovn.tests.functional import base
-
 from neutron.common import utils as n_utils
 from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import l3 as l3_apidef
@@ -28,10 +23,15 @@ from neutron_lib import constants as n_consts
 from neutron_lib.plugins import directory
 from ovsdbapp.backend.ovs_idl import idlutils
 
+from networking_ovn.common import constants as ovn_const
+from networking_ovn.common import utils as ovn_utils
+from networking_ovn.l3 import l3_ovn_scheduler as l3_sched
+from networking_ovn.tests.functional import base
+
 
 class TestRouter(base.TestOVNFunctionalBase):
     def setUp(self):
-        super(TestRouter, self).setUp(ovn_worker=True)
+        super(TestRouter, self).setUp()
         self.chassis1 = self.add_fake_chassis(
             'ovs-host1', physical_nets=['physnet1', 'physnet3'])
         self.chassis2 = self.add_fake_chassis(
@@ -133,28 +133,11 @@ class TestRouter(base.TestOVNFunctionalBase):
     def test_gateway_chassis_with_cms_and_no_bridge_mappings(self):
         # chassis1 is having proper bridge mappings.
         # chassis3 is having enable-chassis-as-gw, but no bridge mappings.
+        # Test if chassis1 is selected as candidate or not.
         self.chassis3 = self.add_fake_chassis(
             'ovs-host3',
             external_ids={'ovn-cms-options': 'enable-chassis-as-gw'})
-        ovn_client = self.l3_plugin._ovn_client
-        ext1 = self._create_ext_network(
-            'ext1', 'vlan', 'physnet1', 1, "10.0.0.1", "10.0.0.0/24")
-        # As we have 'gateways' in the system, but without required
-        # chassis we should not schedule gw in that case at all.
-        self._set_redirect_chassis_to_invalid_chassis(ovn_client)
-        with mock.patch.object(ovn_client._ovn_scheduler, 'select',
-                               return_value=[self.chassis1]), \
-            mock.patch.object(self.l3_plugin.scheduler, 'select',
-                              side_effect=[self.chassis1]):
-            gw_info = {'network_id': ext1['network']['id']}
-            self._create_router('router1', gw_info=gw_info)
-
-        with mock.patch.object(
-                ovn_client._nb_idl, 'update_lrouter_port') as ulrp:
-            self.l3_plugin.schedule_unhosted_gateways()
-            # Make sure that we don't schedule on chassis3
-            # and do not updated the lrp port.
-            ulrp.assert_not_called()
+        self._check_gateway_chassis_candidates([self.chassis1])
 
     def test_gateway_chassis_with_bridge_mappings_and_no_cms(self):
         # chassis1 is configured with proper bridge mappings,
@@ -339,9 +322,9 @@ class TestRouter(base.TestOVNFunctionalBase):
             self._set_redirect_chassis_to_invalid_chassis(ovn_client)
             self.l3_plugin.schedule_unhosted_gateways()
 
-            # We can't test call_count for these mocks, as we have enabled
-            # ovn_worker which will trigger chassis events and eventually
-            # calling schedule_unhosted_gateways
+            # We can't test call_count for these mocks, as we have disabled
+            # maintenance_worker which will trigger chassis events
+            # and eventually calling schedule_unhosted_gateways
             self.assertTrue(client_select.called)
             self.assertTrue(plugin_select.called)
 
@@ -429,7 +412,7 @@ class TestRouter(base.TestOVNFunctionalBase):
         def _get_result_dict():
             sched_info = {}
             for row in self.nb_api.tables[
-                'Logical_Router_Port'].rows.values():
+                    'Logical_Router_Port'].rows.values():
                 for gwc in row.gateway_chassis:
                     chassis = sched_info.setdefault(gwc.chassis_name, {})
                     chassis[gwc.priority] = chassis.get(gwc.priority, 0) + 1
@@ -503,6 +486,6 @@ class TestRouter(base.TestOVNFunctionalBase):
 
         self.l3_plugin.schedule_unhosted_gateways()
         for row in self.nb_api.tables[
-            'Logical_Router_Port'].rows.values():
+                'Logical_Router_Port'].rows.values():
             self.assertEqual(ovn_const.MAX_GW_CHASSIS,
                              len(row.gateway_chassis))
