@@ -76,7 +76,6 @@ class FakeOvsdbNbOvnIdl(object):
         self.delete_address_set = mock.Mock()
         self.update_address_set = mock.Mock()
         self.get_all_chassis_gateway_bindings = mock.Mock()
-        self.get_chassis_gateways = mock.Mock()
         self.get_gateway_chassis_binding = mock.Mock()
         self.get_unhosted_gateways = mock.Mock()
         self.add_dhcp_options = mock.Mock()
@@ -164,6 +163,7 @@ class FakeOvsdbSbOvnIdl(object):
             ('fake', 'fake-dp')
         self.get_chassis_and_physnets = mock.Mock()
         self.get_gateway_chassis_from_cms_options = mock.Mock()
+        self.db_set = mock.Mock()
         self.is_col_present = mock.Mock()
         self.is_col_present.return_value = False
 
@@ -353,6 +353,7 @@ class FakeOvsdbRow(FakeResource):
             'delete': None,
             'delvalue': None,
             'verify': None,
+            'setkey': None,
         }
 
         # Overwrite default attributes and methods.
